# -*- coding: utf-8 -*-
"""Default settings for net_promoter_score.

Override these settings in the Django settings if required.

"""
from django.conf import settings

from net_promoter_score.models import UserScore


def default_display_function(request):
    """Default display function.

    The default display function uses the days_since_user_score to determine
    whether to show NPS to the user. If the interval since the last survey is
    more than the NPS_DISPLAY_INTERVAL, *or* they've never been surveyed, then
    return True.

    Args:
        request: the HttpRequest object relating to the request.

    """
    assert hasattr(request, 'user'), (
        "Missing middleware: "
        "'django.contrib.auth.middleware.AuthenticationMiddleware'"
    )
    interval = UserScore.objects.days_since_user_score(request.user)
    if interval == -1:
        return True
    if interval > NPS_DISPLAY_INTERVAL:
        return True
    return False


# default display interval is 30 days
NPS_DISPLAY_INTERVAL = getattr(settings, 'NPS_DISPLAY_INTERVAL', 30)
# default display function is to show every 30 days, ignoring any user attrs.
NPS_DISPLAY_FUNCTION = getattr(settings, 'NPS_DISPLAY_FUNCTION', default_display_function)



from __future__ import unicode_literals

from django.apps import apps
from django.db import connection
from django.db.migrations.autodetector import MigrationAutodetector
from django.db.migrations.executor import MigrationExecutor
from django.db.migrations.state import ProjectState
from django.test import TestCase


class MigrationsTests(TestCase):

    def test_for_missing_migrations(self):
        """Checks if there're models changes which aren't reflected in migrations."""
        migrations_loader = MigrationExecutor(connection).loader
        migrations_detector = MigrationAutodetector(
            from_state=migrations_loader.project_state(),
            to_state=ProjectState.from_apps(apps)
        )
        if migrations_detector.changes(graph=migrations_loader.graph):
            self.fail(
                'Your models have changes that are not yet reflected '
                'in a migration. You should add them now.'
            )



# -*- coding: utf-8 -*-
from django.contrib import admin

from net_promoter_score.models import UserScore


class UserScoreAdmin(admin.ModelAdmin):

    """Admin model for UserScore objects."""

    list_display = (
        'user',
        'timestamp',
        'score',
        'group',
        'source'
    )
    list_filter = (
        'timestamp',
        'group',
        'source'
    )
    readonly_fields = (
        'user',
        'timestamp',
        'score',
        'group',
        'reason'
    )

admin.site.register(UserScore, UserScoreAdmin)



# -*- coding: utf-8 -*-


from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='UserScore',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('timestamp', models.DateTimeField()),
                ('score', models.IntegerField(default=-1, help_text='0-6=Detractor; 7-8=Neutral; 9-10=Promoter', db_index=True)),
                ('reason', models.TextField(help_text='Reason for the score', blank=True)),
                ('group', models.CharField(default='unknown', help_text='Detractor, neutral or promoter.', max_length=10, db_index=True, choices=[('unknown', 'No answer'), ('detractor', 'Detractor (0-6)'), ('neutral', 'Neutral (7-8)'), ('promoter', 'Promoter (9-10)')])),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
from django import forms

from net_promoter_score.models import UserScore


class UserScoreForm(forms.ModelForm):

    """Form for capturing NPS score."""

    reason = forms.CharField(max_length=512, required=False)

    def __init__(self, *args, **kwargs):
        assert 'user' in kwargs, "Score must have a valid user."
        self.user = kwargs.pop('user')
        super(UserScoreForm, self).__init__(*args, **kwargs)

    class Meta:
        model = UserScore
        fields = ('score', 'reason')
        exclude = ('user', )

    def clean_score(self):
        score = self.cleaned_data['score']
        if score < -1 or score > 10:
            raise forms.ValidationError("Score must be between 0-10")
        return score

    def save(self, commit=True):
        """Set the user attr of the score."""
        score = super(UserScoreForm, self).save(commit=False)
        score.user = self.user
        return score.save()



# -*- coding: utf-8 -*-
from django.conf.urls import url

from net_promoter_score import views

urlpatterns = [
    url(
        r'^score/$',
        views.post_score,
        name="post_score"
    ),
]



# -*- coding: utf-8 -*-
import datetime

from django.contrib.auth import get_user_model
from django.test import TransactionTestCase

from ..models import UserScore, score_group


class UserScoreModelTests(TransactionTestCase):

    def setUp(self):
        self.user = get_user_model().objects.create_user('zoidberg')
        self.user2 = get_user_model().objects.create_user('nixon')

    def test_defaults(self):
        score = UserScore(user=self.user)
        self.assertEqual(score.user, self.user)
        self.assertEqual(score.timestamp, None)
        self.assertEqual(score.score, -1)
        self.assertEqual(score.reason, "")
        self.assertEqual(score.source, "")
        self.assertEqual(score.group, UserScore.GROUP_UNKNOWN)
        score.save()
        self.assertIsNotNone(score.timestamp)
        # confirm that the group is recalculated correctly
        score.score = 1
        score.save()
        self.assertEqual(score.group, UserScore.GROUP_DETRACTOR)

    def test_strings(self):
        unicode_username = "åß∂ƒ©˙∆˚"
        self.user.username = unicode_username
        self.user.save()
        score = UserScore(user=self.user, score=10)
        score.save()
        str(score)

    def test_functions(self):
        self.assertRaises(AssertionError, score_group, None)
        self.assertRaises(AssertionError, score_group, "None")
        self.assertRaises(AssertionError, score_group, -2)
        self.assertRaises(AssertionError, score_group, 11)
        vals = (
            (-1, UserScore.GROUP_UNKNOWN),
            (0, UserScore.GROUP_DETRACTOR),
            (1, UserScore.GROUP_DETRACTOR),
            (2, UserScore.GROUP_DETRACTOR),
            (3, UserScore.GROUP_DETRACTOR),
            (4, UserScore.GROUP_DETRACTOR),
            (5, UserScore.GROUP_DETRACTOR),
            (6, UserScore.GROUP_DETRACTOR),
            (7, UserScore.GROUP_NEUTRAL),
            (8, UserScore.GROUP_NEUTRAL),
            (9, UserScore.GROUP_PROMOTER),
            (10, UserScore.GROUP_PROMOTER),
        )
        for val, group in vals:
            self.assertEqual(score_group(val), group)


class UserScoreQuerySetTests(TransactionTestCase):

    def setUp(self):
        self.user = get_user_model().objects.create_user('zoidberg')
        self.psX = UserScore(user=self.user, score=-1).save()
        self.ps0 = UserScore(user=self.user, score=0).save()
        self.ps1 = UserScore(user=self.user, score=1).save()
        self.ps2 = UserScore(user=self.user, score=2).save()
        self.ps3 = UserScore(user=self.user, score=3).save()
        self.ps4 = UserScore(user=self.user, score=4).save()
        self.ps5 = UserScore(user=self.user, score=5).save()
        self.ps6 = UserScore(user=self.user, score=6).save()
        self.ps7 = UserScore(user=self.user, score=7).save()
        self.ps8 = UserScore(user=self.user, score=8).save()
        self.ps9 = UserScore(user=self.user, score=9).save()
        self.ps10 = UserScore(user=self.user, score=10).save()

    def test_detractors(self):
        detractors = UserScore.objects.detractors()
        self.assertEqual(detractors.count(), 7)
        self.assertTrue(self.ps0 in detractors)
        self.assertTrue(self.ps1 in detractors)
        self.assertTrue(self.ps2 in detractors)
        self.assertTrue(self.ps3 in detractors)
        self.assertTrue(self.ps4 in detractors)
        self.assertTrue(self.ps5 in detractors)
        self.assertTrue(self.ps6 in detractors)

    def test_neutrals(self):
        neutrals = UserScore.objects.neutrals()
        self.assertEqual(neutrals.count(), 2)
        self.assertTrue(self.ps7 in neutrals)
        self.assertTrue(self.ps8 in neutrals)

    def test_promoters(self):
        promoters = UserScore.objects.promoters()
        self.assertEqual(promoters.count(), 2)
        self.assertTrue(self.ps9 in promoters)
        self.assertTrue(self.ps10 in promoters)

    def test_net_promoter_score(self):
        scores = UserScore.objects.all()
        expected = (2.0 - 7.0) / 11 * 100
        self.assertEqual(scores.net_promoter_score(), expected)

    def test_most_recent_user_score(self):
        score = UserScore.objects.most_recent_user_score(self.user)
        self.assertEqual(score, self.ps10)

    def test_days_since_user_score(self):
        score = UserScore.objects.most_recent_user_score(self.user)
        # need to delete all the others
        UserScore.objects.exclude(id=score.id).delete()
        self.assertEqual(UserScore.objects.days_since_user_score(self.user), 0)

        # set the date back 7 days
        today = datetime.date.today()
        score.timestamp = today - datetime.timedelta(days=7)
        score.save()
        self.assertEqual(UserScore.objects.days_since_user_score(self.user), 7)

        # delete all scores for the user
        score.delete()
        self.assertEqual(UserScore.objects.days_since_user_score(self.user), -1)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.test import TransactionTestCase

from ..forms import UserScoreForm


class UserScoreFormTests(TransactionTestCase):

    """Test suite for promoter score forms."""

    def setUp(self):
        self.user = get_user_model().objects.create_user('zoidberg')

    def validate_form(self, data):
        form = UserScoreForm(data=data, user=self.user)
        return form.is_valid()

    def test_clean_valid_score(self):
        for i in range(0, 11):
            self.assertTrue(self.validate_form(data={'score': i}))

    def test_clean_invalid_score(self):
        for i in (-2, 11, "", None):
            self.assertFalse(self.validate_form(data={'score': i}))

    def test_clean_unicode_reason(self):
        data = {'score': 0, 'reason': "√" * 512}
        self.assertTrue(self.validate_form(data=data))

    def test_clean_invalid_reason(self):
        data = {'score': 0, 'reason': "√" * 513}
        self.assertFalse(self.validate_form(data))

    def test_save(self):
        data = {'score': 0, 'reason': "∂ƒ©˙∆˚¬"}
        form = UserScoreForm(data=data, user=self.user)
        score = form.save()
        self.assertIsNotNone(score)
        self.assertEqual(score.user, self.user)
        self.assertEqual(score.score, 0)
        self.assertEqual(score.reason, data['reason'])



"""Django app for managing Net Promoter Score data.

The NPS is a measure of customer loyalty that is measured by asking your
customers a singe question:

    How likely is it that you would recommend our company/product/service
    to a friend or colleague?"

The answer to this question is a number from 0-10 (inclusive). These scores
are then broken out into three distinct groups: 'detractors' (0-6), 'neutral'
(7-8) and 'promoters' (9-10). The NPS is then the difference between the
number of promoters and detractors (as a percentage of the whole population).

For example, if you ask 100 people, and you get the following results:

    detractors: 20
    neutrals:   10
    promoters:  70

Then your NPS is 70 - 20 = 50.

This app is used to store the individual scores, and calculate the NPS based
on these. It does not contain any templates for displaying the question form,
neither does it put any restriction around how often you ask the question, or
to whom. It is up to the app developer to determine how this should work -
each score is timestamped and linked to a Django User object, so you can
easily work out the time elapsed since the last time they were asked.

"""
default_app_config = 'net_promoter_score.apps.NpsConfig'



# -*- coding: utf-8 -*-

from net_promoter_score import settings


def show_nps(request):
    """Return True if the NPS survey should be shown to the request user.

    This is a dynamic function that uses the request object to determine
    whether to display the survey. It defaults to a basic function that
    looks up the last time the user was shown the survey, so that each
    user is only shown it every X days. This should be overridden in the
    Django settings to provide more sophisticated analysis - users with
    different profiles may be surveyed more/less frequently, and you may
    wish to survery based on previous answers.

    """
    return settings.NPS_DISPLAY_FUNCTION(request)



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('net_promoter_score', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='userscore',
            name='source',
            field=models.CharField(help_text="Source of user score, used for filtering results, e.g. 'app', 'web', 'email'.", max_length=20, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.conf import settings
from django.utils.timezone import now as tz_now


def score_group(score):
    """Return 'detractor', 'neutral' or 'promoter'."""
    assert isinstance(score, int), "Score must be an integer value"
    assert score >= -1, "Score cannot be less than -1"
    assert score <= 10, "Score cannot be greater than 10"
    if score == -1:
        return UserScore.GROUP_UNKNOWN
    elif score < 7:
        return UserScore.GROUP_DETRACTOR
    elif score < 9:
        return UserScore.GROUP_NEUTRAL
    else:
        return UserScore.GROUP_PROMOTER


class UserScoreQuerySet(models.query.QuerySet):

    """Custom UserScore queryset - calculates the NPS."""

    def promoters(self):
        """Filter queryset by promoters."""
        return self.filter(group=UserScore.GROUP_PROMOTER)

    def detractors(self):
        """Filter queryset by detractors."""
        return self.filter(group=UserScore.GROUP_DETRACTOR)

    def neutrals(self):
        """Filter queryset by neutrals."""
        return self.filter(group=UserScore.GROUP_NEUTRAL)

    def net_promoter_score(self):
        """Calculate the NPS score from a set of UserScore objects.

        The NPS is the % of promoters - % of detractors (ignoring
        neutrals).

        """
        promoters = self.promoters().count()
        detractors = self.detractors().count()
        neutrals = self.neutrals().count()
        total = promoters + detractors + neutrals
        return (100.0 * (promoters - detractors)) / total

    def most_recent_user_score(self, user):
        """Return the most recent user UserScore."""
        return self.filter(user=user).order_by('-id').first()

    def days_since_user_score(self, user):
        """Return the number of days since the User last submitted a score.

        Returns -1 if the user has never been asked.

        """
        score = self.most_recent_user_score(user)
        if score is None:
            return -1
        else:
            return (tz_now().date() - score.timestamp.date()).days


class UserScore(models.Model):

    """Records the NPS score given by a user."""

    GROUP_UNKNOWN = 'unknown'
    GROUP_PROMOTER = 'promoter'
    GROUP_DETRACTOR = 'detractor'
    GROUP_NEUTRAL = 'neutral'

    GROUP_CHOICES = (
        (GROUP_UNKNOWN, 'No answer'),
        (GROUP_DETRACTOR, 'Detractor (0-6)'),
        (GROUP_NEUTRAL, 'Neutral (7-8)'),
        (GROUP_PROMOTER, 'Promoter (9-10)')
    )

    class Meta:
        app_label = 'net_promoter_score'

    user = models.ForeignKey(settings.AUTH_USER_MODEL, db_index=True)
    timestamp = models.DateTimeField()
    score = models.IntegerField(
        default=-1,
        blank=False, null=False,
        # validators=[MinValueValidator(-1), MaxValueValidator(10)],
        help_text="0-6=Detractor; 7-8=Neutral; 9-10=Promoter",
        db_index=True,
    )
    reason = models.TextField(
        blank=True,
        help_text="Reason for the score"
    )
    # this field is calculated from the score, but is denormalised into
    # a separate property to make it easier to report on. It's kept in
    # sync in the save method.
    group = models.CharField(
        max_length=10,
        default=GROUP_UNKNOWN,
        choices=GROUP_CHOICES,
        help_text="Detractor, neutral or promoter.",
        db_index=True
    )
    source = models.CharField(
        max_length=20,
        blank=True,
        help_text=(
            "Source of user score, used for filtering results, "
            "e.g. 'app', 'web', 'email'."
        )
    )

    objects = UserScoreQuerySet.as_manager()

    def __str__(self):
        if self.timestamp:
            return "{} ({}/10), {}".format(
                self.group,
                self.score,
                self.timestamp.date()
            )
        else:
            return "{} ({}/10), unsaved".format(
                self.group,
                self.score
            )

    def __repr__(self):
        return "<UserScore: id=%s user=%s score=%s>" % (
            self.id,
            self.user.id,
            self.score
        )

    def save(self, *args, **kwargs):
        """Set the timestamp and group attrs."""
        self.timestamp = self.timestamp or tz_now()
        self.group = score_group(self.score)
        super(UserScore, self).save(*args, **kwargs)
        return self

    def json(self):
        """Response JSON-serializable form of the object."""
        assert self.id is not None, "Score must be saved before calling json()."
        return {
            'id': self.id,
            'user': self.user.id,
            'score': self.score,
            'group': self.group
        }



# -*- coding: utf-8 -*-
import json

from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.test import Client, TransactionTestCase, RequestFactory

from ..models import score_group, UserScore
from ..views import post_score


class UserScoreViewTests(TransactionTestCase):

    """Test suite for promoter score views."""

    def setUp(self):
        self.user = get_user_model().objects.create_user(
            'zoidberg', password="foo"
        )
        self.factory = RequestFactory()

    def test_valid_post_201(self):
        """Test that posting valid data returns the score and status_code 201."""
        data = {'score': 0, 'reason': "√" * 512}
        request = self.factory.post('/', data)
        request.user = self.user
        resp = post_score(request)
        self.assertEqual(resp.status_code, 201)
        respj = json.loads(resp.content)
        self.assertEqual(respj['success'], True)
        self.assertEqual(respj['score']['user'], self.user.id)
        self.assertEqual(respj['score']['score'], data['score'])
        self.assertEqual(respj['score']['group'], score_group(data['score']))

    def test_invalid_post_422(self):
        """Test that posting invalid data returns the errors and status_code 422."""
        request = self.factory.post('/', {'score': -2})
        request.user = self.user
        resp = post_score(request)
        self.assertEqual(resp.status_code, 422)
        respj = json.loads(resp.content)
        self.assertEqual(respj['success'], False)
        self.assertEqual(respj['errors'], [['score', 'Score must be between 0-10']])  # noqa

    def test_client_post_anon_302(self):
        """Test posting as an AnonymousUser to the url."""
        client = Client()
        url = reverse('net_promoter_score:post_score')
        resp = client.post(url, data={'score': 0, 'reason': "√" * 512})
        self.assertEqual(resp.status_code, 302)

    def test_client_post_auth_201(self):
        """Test posting as a User to the url."""
        client = Client()
        client.login(username='zoidberg', password='foo')
        url = reverse('net_promoter_score:post_score')
        resp = client.post(url, data={'score': 0, 'reason': "√" * 512})
        self.assertEqual(resp.status_code, 201)
        self.assertEqual(UserScore.objects.count(), 1)



# -*- coding: utf-8 -*-
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.test import TransactionTestCase, RequestFactory

from ..compat import mock
from ..middleware import NPSMiddleware
from ..models import UserScore


class MiddlewareTests(TransactionTestCase):

    """Test suite for middleware."""

    def setUp(self):
        self.factory = RequestFactory()
        self.user = get_user_model().objects.create_user('zoidberg')
        self.score = UserScore(user=self.user, score=10).save()
        self.middleware = NPSMiddleware()

    def test_process_request(self):
        request = self.factory.get('/')
        request.user = self.user
        with mock.patch('net_promoter_score.settings.NPS_DISPLAY_FUNCTION', lambda r: True):
            resp = self.middleware.process_request(request)
            self.assertIsNone(resp)
            self.assertTrue(request.show_nps)

            # verify that unauthenticated users always return False
            request.user = AnonymousUser()
            resp = self.middleware.process_request(request)
            self.assertIsNone(resp)
            self.assertFalse(request.show_nps)



# -*- coding:utf-8 -*-
"""net_promoter_score views."""
from django.contrib.auth.decorators import user_passes_test
from django.http import JsonResponse
from django.views.decorators.http import require_POST

from net_promoter_score.forms import UserScoreForm


@require_POST
@user_passes_test(lambda u: u.is_authenticated())
def post_score(request):
    """POST handler for NPS scores.

    Returns a JSON object, which includes a 'success' key that is True/False.
    If success is True, the response will also contain a 'score' value that
    is the UserScore.json() object.

        {
          "success": True,
          "score": {"id": 1, "user": 1, "score": 0, "group": "detractor"}
        }

    If success is False, the response will contains an 'errors' list:

        {
          "success": False,
          "errors": [["score", "Score must be between 0-10"]]
        }

    """
    form = UserScoreForm(request.POST, user=request.user)
    if form.is_valid():
        score = form.save()
        return JsonResponse({'success': True, 'score': score.json()}, status=201)  # noqa
    else:
        errors = [(k, v[0]) for k, v in list(form.errors.items())]
        return JsonResponse({'success': False, 'errors': errors}, status=422)



# -*- coding: utf-8 -*-
import datetime

from django.contrib.auth import get_user_model
from django.test import TransactionTestCase, RequestFactory

from ..compat import mock
from ..models import UserScore
from ..settings import default_display_function, NPS_DISPLAY_INTERVAL
from ..utils import show_nps


class FunctionTests(TransactionTestCase):

    """Test suite for misc functions."""

    def setUp(self):
        self.factory = RequestFactory()
        self.user = get_user_model().objects.create_user('zoidberg')
        self.score = UserScore(user=self.user, score=10).save()

    def test_display_to_user(self):
        request = self.factory.get('/')
        request.user = self.user
        with mock.patch('net_promoter_score.settings.NPS_DISPLAY_FUNCTION', lambda r: True):
            self.assertTrue(show_nps(request))
        with mock.patch('net_promoter_score.settings.NPS_DISPLAY_FUNCTION', lambda r: False):
            self.assertFalse(show_nps(request))

    def test_default_display_function(self):
        # test that the default function behaves as expected
        request = self.factory.get('/')
        request.user = self.user
        interval_func = UserScore.objects.days_since_user_score
        self.assertEqual(interval_func(self.user), 0)
        self.assertEqual(NPS_DISPLAY_INTERVAL, 30)

        # 0 days since survey - don't show
        self.assertFalse(default_display_function(request))

        # 30 days since survey - don't show
        self.score.timestamp = self.score.timestamp - datetime.timedelta(days=30)
        self.score.save()
        self.assertFalse(interval_func(self.user) > NPS_DISPLAY_INTERVAL)
        self.assertFalse(default_display_function(request))

        # 31 days since survey - *do* show
        self.score.timestamp = self.score.timestamp - datetime.timedelta(days=31)
        self.score.save()
        self.assertTrue(interval_func(self.user) > NPS_DISPLAY_INTERVAL)
        self.assertTrue(default_display_function(request))

        # never been surveyed - *do* show
        self.score.delete()
        self.assertEqual(interval_func(self.user), -1)
        self.assertTrue(default_display_function(request))



import os
from setuptools import setup, find_packages

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name="django-nps",
    version="0.4.2",
    packages=find_packages(),
    install_requires=['Django>=1.8'],
    include_package_data=True,
    description='Django app supporting Net Promoter Score (NPS) surveys.',
    license='MIT',
    long_description=README,
    url='https://github.com/yunojuno/django-nps',
    author='Hugo Rodger-Brown',
    author_email='hugo@yunojuno.com',
    maintainer='Hugo Rodger-Brown',
    maintainer_email='hugo@yunojuno.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



# -*- coding: utf-8 -*-
"""NPS middleware used to set request attrs and cookies.

When determining whether to display the NPS survey to user, we cache
the output (True|False) so that we don't have to do a database lookup
on each request. This value is added to the user session (so a max
of one lookup per session.

"""
try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    # Fallback for Django < 1.10
    MiddlewareMixin = object


from net_promoter_score.utils import show_nps


class NPSMiddleware(MiddlewareMixin):

    """Add show_nps attr to the user session."""

    def process_request(self, request):
        # force instantiation of the request.user SimpleLazyObject
        assert hasattr(request, 'user'), (
            "Missing middleware: "
            "'django.contrib.auth.middleware.AuthenticationMiddleware'"
        )
        if request.user.is_authenticated():
            request.show_nps = show_nps(request)
        else:
            request.show_nps = False
        return None
